@@ -255,8 +255,9 @@ gboolean
 gsd_sound_manager_start (GsdSoundManager *manager,
                          GError **error)
 {
-        char *p, **ps, **k;
-        const char *env, *dd;
+        guint i;
+        const gchar * const * dirs;
+        char *p;
 
         g_debug ("Starting sound manager");
         gnome_settings_profile_start (NULL);
@@ -266,30 +267,18 @@ gsd_sound_manager_start (GsdSoundManager *manager,
 
         /* ... and we listen to changes of the theme base directories
          * in $HOME ...*/
+        p = g_build_filename (g_get_user_data_dir (), "sounds", NULL);
+        register_directory_callback (manager, p, NULL);
+        g_free (p);
 
-        if ((env = g_getenv ("XDG_DATA_HOME")) && *env == '/')
-                p = g_build_filename (env, "sounds", NULL);
-        else if (((env = g_getenv ("HOME")) && *env == '/') || (env = g_get_home_dir ()))
-                p = g_build_filename (env, ".local", "share", "sounds", NULL);
-        else
-                p = NULL;
-
-        if (p) {
+        /* ... and globally. */
+        dirs = g_get_system_data_dirs ();
+        for (i = 0; dirs[i] != NULL; i++) {
+                p = g_build_filename (dirs[i], "sounds", NULL);
                 register_directory_callback (manager, p, NULL);
                 g_free (p);
         }
 
-        /* ... and globally. */
-        if (!(dd = g_getenv ("XDG_DATA_DIRS")) || *dd == 0)
-                dd = "/usr/local/share:/usr/share";
-
-        ps = g_strsplit (dd, ":", 0);
-
-        for (k = ps; *k; ++k)
-                register_directory_callback (manager, *k, NULL);
-
-        g_strfreev (ps);
-
         gnome_settings_profile_end (NULL);
 
         return TRUE;
